"""
A class representing an initial value problem (IVP), specifically of the form:
    
    du/dt = f(u,t,p)   for u(tI) = uI

In which the problem would be solved from t=tI to tF
And p are a set of parameters (fixed for all time)

"""

class IVP():
    def __init__(self, uI, tI, tF, p = {}):
        """

        Args:
            uI (float list): initial condition of state.
            tI (float): initial time.
            tF (float): final time.
            p (dictionary): set of fixed parameters. Optional.
        """
        self.uI = uI[:]
        self.tI = tI
        self.tF = tF
        self.p  = p
        
    def evalf(self,u,t):
        """    

        Args:
            u (float list): current solution.
            t (float): current time.

        Returns:
            float list: f(u,t).

        """
        raise NotImplementedError("evalf is not implemented for this object") 
    
    def get_tI(self):
        """        

        Returns:
            float: initial time.

        """
        return self.tI

    def get_tF(self):
        """

        Returns:
            float: final time.

        """
        return self.tF
    
    def get_uI(self):
        """        

        Returns:
            float list: initial state.

        """
        return self.uI
    
    def get_p(self,name):
        """
      
        Arg:
            name (key): a key which should be in the object's parameter 
            dictionary 

        Returns:
            value of parameter key given by name

        """

        return self.p[name]
 
### step_FE, solve, etc. part of pset 1 ###