################################################################################
# Model predator-prey population dynamics
#
#    dr/dt =  a r - b r f
#    df/dt = -m f + c b r f 
#
# where: r  = population of prey (e.g. number of rabbits)
#        f  = population of predator (e.g. number of foxes)
#        a  = birth rate of prey per capita
#        m  = mortality rate of predator per capita
#        b  = predation rate per prey x predator
#        c  = frequency at which predation results prey
#

import matplotlib.pyplot as plt
import IVP_predprey   

def solve_predprey(predprey_IVP, dt):
    # Sets initial condition
    t0 = predprey_IVP.get_tI()
    v0 = predprey_IVP.get_uI() # v0 is a float list!

    t = [t0]
    v = [v0] # v is a list of float lists with v0 the first list!
    
    # Loop from from t=t0 to t>=tF
    tn = t[0]
    vn = v[0]

    while (tn<tF):
        # Calculate forcing
        fn = predprey_IVP.evalf(vn,tn)
        
        # FORWARD EULER
        # Update solution and time
        vn1 = []
        for i in range(len(vn)):
            vn1.append(vn[i] + dt*fn[i])

        tn1 = tn + dt
       
        # Append to v and t
        v.append(vn1)
        t.append(tn1) # store time in minutes
        
        # Set vn and tn for next iteration
        vn = vn1
        tn = tn1
    
    return t, v


p = {}
p['a'] = 2.
p['b'] = 0.01
p['c'] = 0.1
p['m'] = 1.

rI = 500.
fI = 100.
uI = [rI, fI]        
tF = 10. # final time to simulate to (month)
dt = 0.01 # time increment to give solutions at (s)
        

# Initialize PredPreyIVP object
predprey_IVP = IVP_predprey.PredPreyIVP(uI, 0.0, tF, p)

# Solve predprey IVP
t, v = solve_predprey(predprey_IVP, dt)

# Unpack the rabbit and fox states into separate arrays for plotting
r = []
f = []
for vn in v:
    r.append(vn[0])
    f.append(vn[1])

# Plot   
plt.plot(t,r,label='rabbit')
plt.plot(t,f,label='fox')
plt.xlabel('t (months)')
plt.ylabel('population size')
plt.grid(True)
plt.legend()
