################################################################################
# 6.S082 Spring 2021
# tests_home_heating

import numpy as np
import matplotlib.pyplot as plt
from scipy import linalg
import IVP_numpy
from mysolve import mysolve


    
################################################################################
## test_IVP
# Solution to the system of ODE, time evolution
################################################################################

class Home_heating(IVP_numpy.IVP_numpy):
    
    def evalf(self, u, t):

        kb = self.get_p('kb')
        kA = self.get_p('kA')
        
        f = np.matmul(kA,u) + kb
        
        return f

def run_test_IVP(method):
    
    
    tI = 0.0
    tF = 10.0            # hours
    
    k = 0.5              # Newton cooling constant (1/h)
    u_outside = -3.0     # outside air temperature (Celsius)
    u_ground = 2.0       # near-surface ground temperature (Celsius)
    uI = np.array([u_outside, u_outside, u_ground])    
    u_boundary = np.array([u_outside, u_outside, u_ground])
    
    power_per_room_heater = 10.0
    heaters_on = np.array([1.0,0.0,0.0])
    
    power_per_room_GSHP = 6.0       # Ground-source heat pump
    COP_GSHP = 3.0      # Coefficient of performance
    GSHP_on = np.array([0.0,1.0,0.0])
    
    kb_heaters = power_per_room_heater*heaters_on
    kb_GSHP = COP_GSHP*power_per_room_GSHP*GSHP_on
    
    kb = k*u_boundary + kb_heaters + kb_GSHP
    total_power = np.sum(power_per_room_heater*heaters_on + power_per_room_GSHP*GSHP_on)
    
    A = np.array( [[-2.0, 1.0, 0.0], [1.0, -3.0, 1.0], [0.0, 1.0, -2.0]] )
    kA = k*A
    
    p = {}
    p['kb'] = kb
    p['kA'] = kA
    
    this_IVP = Home_heating(uI, tI, tF, p)
    
    dt = 2e-1
    t, u = IVP_numpy.solve(this_IVP, dt, method)
    
    plt.close('all')
    fig = plt.figure()
    plt.plot(t, u[:,0],'g-',label='Annex')
    plt.plot(t, u[:,1],'r-',label='Main')
    plt.plot(t, u[:,2],'b-',label='Basement')
    plt.xlabel('Time (hours)')
    plt.ylabel('Temperature (deg C)')
    plt.grid(True)
    plt.legend()
    s = 'Total power: ' + format(total_power,'.3e')
    
    return s





################################################################################
## test_linalg
# Solution to the system of ODE, equilibrium
################################################################################

def run_test_linalg(method):
    
        
    k = 0.5              # Newton cooling constant
    u_outside = -3.0     # outside air temperature (Celsius)
    u_ground = 2.0       # near-surface ground temperature (Celsius)
    uI = np.array([u_outside, u_outside, u_ground])    
    u_boundary = np.array([u_outside, u_outside, u_ground])
    
    power_per_room_heater = 10.0
    heaters_on = np.array([1.0,0.0,0.0])
    
    power_per_room_GSHP = 6.0       # Ground-source heat pump
    COP_GSHP = 3.0      # Coefficient of performance
    GSHP_on = np.array([0.0,1.0,0.0])
    
    kb_heaters = power_per_room_heater*heaters_on
    kb_GSHP = COP_GSHP*power_per_room_GSHP*GSHP_on
    
    kb = k*u_boundary + kb_heaters + kb_GSHP
    total_power = np.sum(power_per_room_heater*heaters_on + power_per_room_GSHP*GSHP_on)
    
    A = np.array( [[-2.0, 1.0, 0.0], [1.0, -3.0, 1.0], [0.0, 1.0, -2.0]] )
    kA = k*A
    
    u = linalg.solve(kA,-kb)
    my_u = mysolve(kA,-kb)
    my_err = max(abs(my_u-u))
    s = ('Solution: ' + np.array2string(u) + 
         '       Discrepancy between solvers = ' + format(my_err,'.3e'))
 
    return s
    


################################################################################
## Main script
################################################################################

if __name__ == "__main__":
    
    
    # You may change the following list to set the tests you would like to run     
    tests_to_run = [run_test_linalg, run_test_IVP]
    
    # You may change the following list to set the methods you would like to run on each test     
    methods_to_test = [IVP_numpy.step_FE]

    
    for test in tests_to_run:
        print()
        print(test.__name__,'results:')
        for method in methods_to_test:
            print(test(method))
            
    
